import { createApi } from '@reduxjs/toolkit/query/react'
import { gql } from 'graphql-request'
import { graphqlRequestBaseQuery } from '@rtk-query/graphql-request-base-query'

import { useSelector, useDispatch } from 'react-redux'







export function jwtDecode(token) {                         // расщифровки токена авторизации
    try {
        const tokenArr = token.split(".");
        const tokenJsonStr = atob(tokenArr[1]);
        const tokenJson = JSON.parse(tokenJsonStr);
        return tokenJson;
    }
    catch (error) { }
}

const prepareHeaders =  (headers, { getState }) => {
    // By default, if we have a token in the store, let's use that for authenticated requests
     const token = getState().auth.token
    //const token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOnsiaWQiOiI2MjYyOTA1ZWNiZTY5ODVlZWQ0NzIzMDAiLCJsb2dpbiI6InRlc3QiLCJhY2wiOlsiNjI2MjkwNWVjYmU2OTg1ZWVkNDcyMzAwIiwidXNlciJdfSwiaWF0IjoxNjc5MzM1NjQyfQ.k51I9BFbNit85SCkZGU3HO5c_G159VsO3GFxaj0unTc"


    if (token) {
        headers.set("Authorization", `Bearer ${token}`);
    }
    return headers;
}


export const api = createApi({
    baseQuery: graphqlRequestBaseQuery({
        url: "http://player.node.ed.asmer.org.ua/graphql",
        prepareHeaders
    }),
    endpoints: (builder) => builder.query({
        getLogin : builder.mutation({
            query: ({login,password})=>({
                document: gql`query getLogin($login: String!, $password: String!){
                    login(login: $login, password: $password)
                } `,
                variables: {login, password}
            }),  
        }),
        getRegistr: builder.mutation({
            query: ({login,password})=>({
                document: gql`mutation registrUser($login: String!,$password: String!){
                       createUser(login:$login,password: $password){
                       _id login
                       }
                     } `,
                variables: {  login,password }
            }),  
        }),
        getTracks: builder.query({
            query: () => ({
                document: gql`query trackFind {
                    TrackFind(query:"[{}]"){
                       _id url originalFileName owner{_id} 
                       }
                   }`
                }),
        }),
        getPlaylistOne: builder.query({
            query: (_id) => ({
                document: gql` query playlistFindOne($q: String) {
                    PlaylistFindOne(query: $q){
                      _id name description tracks{
                        _id url originalFileName owner{_id} id3{title artist genre trackNumber} playlists{
                            _id owner{_id} description } 
                      }
                    }
                  }`,
                  variables : {q : JSON.stringify([{_id}])}
                }),
        }),
        getPlaylist: builder.query({
            query: () => ({
                document: gql`query playlistFind {
                        PlaylistFind (query:"[{}]"){
                           _id name owner{_id login nick} description tracks{
                            _id url originalFileName owner{_id login avatar{url}} playlists{_id} }
                        }
                      }`
                }),
        }),
        getImages: builder.query({
            query: () => ({
                document: gql` query imageFind {
                        ImageFind (query:"[{}]"){
                          _id text url
                          originalFileName
                          userAvatar{
                            _id
                          }
                          owner{
                            _id
                          }
                        }
                      }`
            }),  
        }),
        getUser: builder.query({
            query: (_id) => ({
                document: gql `query userFindOne($q: String) {
                    UserFindOne(query : $q){
                      _id nick  createdAt login avatar{
                              _id url
                            }
                      }
                  }`,
                  variables : {q : JSON.stringify([{_id}])}
            }),  
        }),
    }),
})

export const {  useGetLoginMutation,useGetRegistrMutation, useGetTracksQuery,useGetPlaylistQuery,useGetPlaylistOneQuery,
    useGetImagesQuery,useGetUserQuery } = api
console.log(api)






export const baseURL = 'http://player.node.ed.asmer.org.ua/'

// export const LeftMenu = () => {
//     const hookResult = useGetImagesQuery()
//     const {data, isFetching} = hookResult
//     console.log(hookResult, data, isFetching)
//     if (isFetching)
//         return <h2>LOADING</h2>
//     return (
//         <ul>
            
//             {data.ImageFind.map(cat => <li key={cat?._id}><audio controls src={baseURL + cat?.url}></audio></li>)}
//         </ul>
//     )
// }



// export const counterSlice = createSlice({
//     name : "counter",
//     initialState : {value:0},
//     reducers: {
//         increment: (state) => { state.value += 1 },
//         decrement: (state) => { state.value -= 1 },
//         incrementAmount: (state,action) => {
//             state.value += action.payload
//         }
//     },
// })

// export const { increment,decrement,incrementAmount } = counterSlice.actions

// // export default counterSlice.reducer

// export const store = configureStore({
//     reducer: {
//         counter : counterSlice.reducer
//     }
// })

// export function Counter (){
//     const count = useSelector((state) => state.counter.value)
//     const dispatch = useDispatch()

//     return (
//         <div>
//       <div>
//         <button
//           aria-label="Increment value"
//           onClick={() => dispatch(increment())}
//         >
//           Increment
//         </button>
//         <span>{count}</span>
//         <button
//           aria-label="Decrement value"
//           onClick={() => dispatch(decrement())}
//         >
//           Decrement
//         </button>
//       </div>
//     </div>
//     )
// }